# -*- coding: utf-8 -*-
"""
Core client, used for all API requests.
"""
from plivo.resources import (Accounts, Addresses, Applications, Calls,
                             Conferences, Endpoints, Identities, Messages,
                             Numbers, Pricings, Recordings, Subaccounts)
from plivo.resources.live_calls import LiveCalls
from plivo.rest.base_client import BaseClient


class Client(BaseClient):
    def __init__(self, auth_id=None, auth_token=None, proxies=None, timeout=5):
        """
        The Plivo API client.

        Deals with all the API requests to be made.
        """
        BaseClient.__init__(self, auth_id, auth_token, proxies, timeout)

        self.account = Accounts(self)
        self.subaccounts = Subaccounts(self)
        self.applications = Applications(self)
        self.calls = Calls(self)
        self.live_calls = LiveCalls(self)
        self.conferences = Conferences(self)
        self.endpoints = Endpoints(self)
        self.messages = Messages(self)
        self.numbers = Numbers(self)
        self.pricing = Pricings(self)
        self.recordings = Recordings(self)
        self.addresses = Addresses(self)
        self.identities = Identities(self)



# -*- coding: utf-8 -*-
"""
Phlo class
"""
from plivo.utils import to_param_dict

from plivo.resources.nodes import Node, MultiPartyCall, Member
from plivo.base import PlivoResource, PlivoResourceInterface
from plivo.utils.validators import *


class Phlo(PlivoResource):
    _name = 'Phlo'
    _identifier_string = 'phlo_id'

    def node(self, node_type, node_id):
        self.node_id = node_id
        return self.client.request(
            'GET', ('phlo', self.phlo_id, node_type, node_id), response_type=Node)

    def multi_party_call(self, node_id):
        self.node_id = node_id
        self.node_type = 'multi_party_call'
        return self.client.request(
            'GET', ('phlo', self.phlo_id, self.node_type, node_id), response_type=MultiPartyCall)

    def run(self, **kwargs):
        return self.client.request('POST', ('account', self.client.session.auth[0], 'phlo', self.phlo_id),
                                   to_param_dict(self.run, kwargs, func_args_check=False))


class Phlos(PlivoResourceInterface):
    _resource_type = Phlo

    def __init__(self, client):
        super(Phlos, self).__init__(client)

    @validate_args(phlo_id=[of_type(six.text_type)])
    def get(self, phlo_id):
        self.phlo_id = phlo_id
        return self.client.request(
            'GET', ('phlo', phlo_id), response_type=Phlo)



# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface
from ..exceptions import *
from ..utils import *


class Message(PlivoResource):
    _name = 'Message'
    _identifier_string = 'message_uuid'

    def get(self):
        return self.client.messages.get(self.id)

    def delete(self):
        raise InvalidRequestError('Cannot delete a Message resource')

    def update(self):
        raise InvalidRequestError('Cannot update a Message resource')


class Messages(PlivoResourceInterface):
    _resource_type = Message

    @validate_args(
        src=[is_phonenumber()],
        dst=[is_iterable(of_type(six.text_type), '<')],
        text=[of_type(six.text_type)],
        type_=[optional(all_of(of_type(six.text_type), is_in(('sms', ))))],
        url=[optional(is_url())],
        method=[optional(of_type(six.text_type))],
        log=[optional(of_type_exact(bool))])
    def create(self,
               src,
               dst,
               text,
               type_='sms',
               url=None,
               method='POST',
               log=True):
        if src in dst.split('<'):
            raise ValidationError(
                'destination number cannot be same as source number')
        return self.client.request('POST', ('Message', ),
                                   to_param_dict(self.create, locals()))

    @validate_args(message_uuid=[of_type(six.text_type)])
    def get(self, message_uuid):
        return self.client.request(
            'GET', ('Message', message_uuid), response_type=Message)

    @validate_args(
        subaccount=[optional(is_subaccount())],
        message_direction=[optional(is_in(('inbound', 'outbound')))],
        message_time__gt=[optional(is_valid_date())],
        message_time__gte=[optional(is_valid_date())],
        message_time__lt=[optional(is_valid_date())],
        message_time__lte=[optional(is_valid_date())],
        message_time=[optional(is_valid_date())],
        message_state=[
            optional(
                is_in(('queued', 'sent', 'failed', 'delivered', 'undelivered',
                       'rejected')))
        ],
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self,
             subaccount=None,
             message_direction=None,
             message_time__gt=None,
             message_time__gte=None,
             message_time__lt=None,
             message_time__lte=None,
             message_time=None,
             message_state=None,
             limit=None,
             offset=None,
             error_code=None):
        return self.client.request(
            'GET', ('Message', ),
            to_param_dict(self.list, locals()),
            response_type=ListResponseObject,
            objects_type=Message)



import six

from plivo.xml import PlivoXMLElement, map_type


class SpeakElement(PlivoXMLElement):
    _name = 'Speak'
    _nestable = []

    @property
    def voice(self):
        return self.__voice

    @voice.setter
    def voice(self, value):
        self.__voice = six.text_type(value) if value is not None else None

    def set_voice(self, value):
        self.voice = value
        return self

    @property
    def language(self):
        return self.__language

    @language.setter
    def language(self, value):
        self.__language = six.text_type(value) if value is not None else None

    def set_language(self, value):
        self.language = value
        return self

    @property
    def loop(self):
        return self.__loop

    @loop.setter
    def loop(self, value):
        self.__loop = int(value) if value is not None else None

    def set_loop(self, value):
        self.loop = value
        return self

    def __init__(
            self,
            content,
            voice=None,
            language=None,
            loop=None,
    ):
        super(SpeakElement, self).__init__()

        self.content = content
        self.voice = voice
        self.language = language
        self.loop = loop

    def to_dict(self):
        d = {
            'voice': self.voice,
            'language': self.language,
            'loop': self.loop,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



import six

from plivo.xml import (PlayElement, PlivoXMLElement, SpeakElement, WaitElement,
                       map_type)


class PreAnswerElement(PlivoXMLElement):
    _name = 'PreAnswer'
    _nestable = ['Speak', 'Play', 'Wait']

    def __init__(
            self, ):
        super(PreAnswerElement, self).__init__()

        pass

    def to_dict(self):
        d = {}
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }

    def add_speak(
            self,
            content,
            voice=None,
            language=None,
            loop=None, ):
        self.add(
            SpeakElement(
                content=content,
                voice=voice,
                language=language,
                loop=loop, ))
        return self

    def add_play(
            self,
            content,
            loop=None, ):
        self.add(PlayElement(
            content=content,
            loop=loop, ))
        return self

    def add_wait(
            self,
            length=None,
            silence=None,
            min_silence=None,
            beep=None, ):
        self.add(
            WaitElement(
                length=length,
                silence=silence,
                min_silence=min_silence,
                beep=beep, ))
        return self
